import io
from gmshparser.mesh import Mesh
from gmshparser.node import Node
from gmshparser.node_entity import NodeEntity
from gmshparser.element import Element
from gmshparser.element_entity import ElementEntity
from gmshparser.helpers import parse_ints, parse_floats, get_triangles


def test_parse_ints():
    s = io.StringIO("1 2 3 4")
    assert parse_ints(s) == [1, 2, 3, 4]


def test_parse_floats():
    s = io.StringIO("1.1 2.2 3.3 4.4")
    assert parse_floats(s) == [1.1, 2.2, 3.3, 4.4]


def test_get_triangles():
    mesh = Mesh()

    n1 = Node()
    n1.set_tag(1)
    n1.set_coordinates((0.0, 0.0, 0.0))

    n2 = Node()
    n2.set_tag(2)
    n2.set_coordinates((1.0, 0.0, 0.0))

    n3 = Node()
    n3.set_tag(3)
    n3.set_coordinates((0.0, 1.0, 0.0))

    n4 = Node()
    n4.set_tag(4)
    n4.set_coordinates((1.0, 2.0, 3.0))

    ne1 = NodeEntity()
    ne1.set_dimension(2)
    ne1.set_tag(1)
    ne1.add_node(n1)
    ne1.add_node(n2)
    ne1.add_node(n3)
    ne1.add_node(n4)

    e1 = Element()
    e1.set_tag(1)
    e1.set_connectivity([1, 2, 3])

    ee1 = ElementEntity()
    ee1.set_dimension(2)
    ee1.set_tag(1)
    ee1.set_element_type(2)
    ee1.add_element(e1)

    ee2 = ElementEntity()
    ee2.set_dimension(1)
    ee2.set_element_type(3)

    mesh.add_node_entity(ne1)
    mesh.add_element_entity(ee1)
    mesh.add_element_entity(ee2)

    (X, Y, T) = get_triangles(mesh)
    assert X == [0.0, 1.0, 0.0]
    assert Y == [0.0, 0.0, 1.0]
    assert T == [[0, 1, 2]]



from gmshparser.cli import main
from io import StringIO
import os

__content__ = """
$MeshFormat
4.1 0 8
$EndMeshFormat
$Nodes
1 6 1 6
2 1 0 6
1
2
3
4
5
6
0. 0. 0.
1. 0. 0.
1. 1. 0.
0. 1. 0.
2. 0. 0.
2. 1. 0.
$EndNodes
$Elements
1 2 1 2
2 1 3 2
1 1 2 3 4
2 2 5 6 3
$EndElements
$NodeData
1
"A scalar view"
1
0.0
3
0
1
6
1 0.0
2 0.1
3 0.2
4 0.0
5 0.2
6 0.4
$EndNodeData
"""


def test_main(tmpdir):
    fh = tmpdir.join("mesh1.msh")
    fh.write(__content__.strip())
    filename = os.path.join(fh.dirname, fh.basename)
    # We just check that there comes some print from functions to stdout

    output = StringIO()
    main([filename, "info"], output)
    print(output.getvalue())
    assert len(output.getvalue()) > 100

    output = StringIO()
    main([filename, "nodes"], output)
    assert len(output.getvalue()) == 176

    output = StringIO()
    main([filename, "elements"], output)
    assert len(output.getvalue()) == 26



import sys
import argparse
from . import parse

"""gmshparser cli provides some helpers to convert mesh to another format."""


def info(mesh, file) -> None:
    print("---- MESH SUMMARY ----", file=file)
    print(mesh, file=file)


def nodes(mesh, file) -> None:
    print(mesh.get_number_of_nodes(), file=file)
    for entity in mesh.get_node_entities():
        for node in entity.get_nodes():
            nid = node.get_tag()
            x, y, z = node.get_coordinates()
            print("%d %f %f %f" % (nid, x, y, z), file=file)


def elements(mesh, file) -> None:
    print(mesh.get_number_of_elements(), file=file)
    for entity in mesh.get_element_entities():
        eltype = entity.get_element_type()
        for element in entity.get_elements():
            elid = element.get_tag()
            elcon = " ".join(map(str, element.get_connectivity()))
            print("%s %s %s" % (elid, eltype, elcon), file=file)


def main(argv=None, file=sys.stdout) -> None:
    parser = argparse.ArgumentParser()
    choices = {"info": info, "nodes": nodes, "elements": elements}
    parser.add_argument('filename', action='store')
    parser.add_argument('action', choices=list(choices.keys()))
    args = parser.parse_args(argv or sys.argv[1:])
    mesh = parse(args.filename)
    choices[args.action](mesh, file)
